@@ -88,7 +88,7 @@ def strat_daily(y, btc='BTC',tone=15,ttwo=26, p1=1., p2=1.05, long_expo=1., shor
             S[t] = short_expo
     return S
 
-def compute_daily_signal_lo(df=None, core_tokens=['BTC', 'ETH'], extra_tokens=['AVAX'],tone=15, ttwo=26, p1btc = 1.,p2btc = 1.05,p1eth = 1.,p2eth = 1.05,lo_sig_suffix='LO',compute_lo_strat=True, plot_html=False, whole_kpis = False):
+def compute_daily_signal_lo(df=None, core_tokens=['BTC', 'ETH'], extra_tokens=['AVAX'],tone=15, ttwo=26, p1btc = 1.,p2btc = 1.05,p1eth = 1.,p2eth = 1.05,lo_sig_suffix='LO',compute_lo_strat=True, plot_html=False, whole_kpis = False, local_root_directory='none'):
     all_tokens = core_tokens + extra_tokens
 
     for me_token in all_tokens:
@@ -201,7 +201,150 @@ def compute_daily_signal_lo(df=None, core_tokens=['BTC', 'ETH'], extra_tokens=['
     return fin_df.copy(), kpi_df
 
 
-def compute_alpha_signal_optimal_lo(df=None, core_tokens=['BTC', 'ETH'], extra_tokens=['AVAX'],     p1btc = 1.01, p2btc = 1.05, p1eth = 1.01, p2eth = 1.05, tone = 14,ttwo = 23, rebalancing_day=6,lo_sig_suffix='LO',compute_lo_strat=True, plot_html=False, whole_kpis=False):
+def compute_daily_signal_rsilo(df=None, core_tokens=['BTC', 'ETH'], rsi_lookback = 8 ,threshold = 60 , lo_sig_suffix='RSILO', compute_lo_strat=True, plot_html=False, local_root_directory='None'):
+    assert len(core_tokens) == 2
+    first_token = core_tokens[0]
+    second_token = core_tokens[1]
+    df['hour'] = df.index.hour
+    all_dfs = None
+    for me_hour in [0, 2, 4, 6, 8, 10, 12, 14, 16, 18, 20, 22]:
+        daily_df = df[df['hour'] == me_hour].copy()
+        daily_df[f'DAILY_RSI_{first_token}_{me_hour}'] = talib.RSI(daily_df[f'close_{first_token}'], rsi_lookback)
+        daily_df[f'DAILY_RSI_{second_token}_{me_hour}'] = talib.RSI(daily_df[f'close_{second_token}'], rsi_lookback)
+        if all_dfs is None:
+            all_dfs = daily_df.copy()
+        else:
+            all_dfs = pd.concat([all_dfs.copy(), daily_df.copy()])
+
+    all_dfs = all_dfs.sort_index()
+    column_list_btc = []
+    column_list_eth = []
+    for me_hour in [0, 2, 4, 6, 8, 10, 12, 14, 16, 18, 20, 22]:
+        all_dfs[f'DAILY_RSI_{first_token}_{me_hour}'] = all_dfs[f'DAILY_RSI_{first_token}_{me_hour}'].fillna(0.)
+        all_dfs[f'DAILY_RSI_{second_token}_{me_hour}'] = all_dfs[f'DAILY_RSI_{second_token}_{me_hour}'].fillna(0.)
+        column_list_btc.append(f'DAILY_RSI_{first_token}_{me_hour}')
+        column_list_eth.append(f'DAILY_RSI_{second_token}_{me_hour}')
+
+    all_dfs[f'DAILY_RSI_{first_token}'] = all_dfs[column_list_btc].sum(axis=1)
+    all_dfs[f'DAILY_RSI_{second_token}'] = all_dfs[column_list_eth].sum(axis=1)
+
+    def compute_dual_RSI(row_df, threshold=60, first_token='BTC', second_token='ETH',lo_sig_suffix='RSILO'):
+        RSI_BTC = row_df[f'DAILY_RSI_{first_token}']
+        RSI_ETH = row_df[f'DAILY_RSI_{second_token}']
+        if RSI_BTC > threshold or RSI_ETH > threshold:
+            if RSI_BTC > RSI_ETH:
+                return {
+                    f'S{lo_sig_suffix}{first_token}': 1,
+                    f'S{lo_sig_suffix}{second_token}': 0
+                }
+            else:
+                return {
+                    f'S{lo_sig_suffix}{first_token}': 0,
+                    f'S{lo_sig_suffix}{second_token}': 1
+                }
+        else:
+            return {
+                f'S{lo_sig_suffix}{first_token}': 0,
+                f'S{lo_sig_suffix}{second_token}': 0
+            }
+
+    go_rsi = lambda x: compute_dual_RSI(x, threshold=threshold, first_token=first_token, second_token=second_token, lo_sig_suffix=lo_sig_suffix)
+    all_dfs['signal_gen'] = all_dfs.apply(go_rsi, axis=1)
+    all_dfs['signal'] = all_dfs['signal_gen'].shift()
+    all_dfs = all_dfs.iloc[1:]
+
+    sig_df = pd.DataFrame().from_records(all_dfs['signal'].values)
+    sig_df.index = all_dfs.index
+
+    df[f'return_{first_token}'] = df[f'close_{first_token}'].pct_change()
+    df[f'return_{second_token}'] = df[f'close_{second_token}'].pct_change()
+
+    data_df = pd.merge(df.copy(), sig_df.copy(), how='left', left_index=True, right_index=True)
+    data_df[f'S{lo_sig_suffix}{first_token}'] = data_df[f'S{lo_sig_suffix}{first_token}'].ffill()
+    data_df[f'S{lo_sig_suffix}{second_token}'] = data_df[f'S{lo_sig_suffix}{second_token}'].ffill()
+
+    ### we rebalance monday morning
+    def get_filter_date(row):
+        ##### to get the proper daily close
+        #### we only keep the signal that has been emitted the day after
+        if row['hour'] == 0:
+            return True
+        else:
+            return False
+
+    data_df['hour'] = data_df.index.hour
+    data_df['rebalancing'] = data_df.apply(get_filter_date, axis=1)
+
+    ##### we rebalance with the close of rebalancing_day :
+    def rebalancing_signal_cut(row, token='BTC', lo_sig_suffix='RSILO'):
+        if row['rebalancing']:
+            return row[f'S{lo_sig_suffix}{token}']
+        else:
+            return np.nan
+
+    goBTC = lambda x: rebalancing_signal_cut(x, token=first_token,lo_sig_suffix=lo_sig_suffix)
+    data_df[f'S{lo_sig_suffix}{first_token}'] = data_df.apply(goBTC, axis=1)
+    goETH = lambda x: rebalancing_signal_cut(x, token=second_token,lo_sig_suffix=lo_sig_suffix)
+    data_df[f'S{lo_sig_suffix}{second_token}'] = data_df.apply(goETH, axis=1)
+
+    data_df[f'S{lo_sig_suffix}{first_token}'] = data_df[f'S{lo_sig_suffix}{first_token}'].ffill()
+    data_df[f'S{lo_sig_suffix}{second_token}'] = data_df[f'S{lo_sig_suffix}{second_token}'].ffill()
+
+    moderation = 1.
+    mod_data_df = data_df.copy()
+
+    def compute_dual_return(row, moderation=1., first_token='BTC', second_token='ETH', lo_sig_suffix='RSILO'):
+        return row[f'S{lo_sig_suffix}{first_token}'] * row[f'return_{first_token}'] * moderation + row[f'S{lo_sig_suffix}{second_token}'] * row[
+            f'return_{second_token}'] * moderation
+
+    go_compute = lambda x: compute_dual_return(x, moderation=moderation, first_token=first_token,second_token=second_token,lo_sig_suffix=lo_sig_suffix)
+    mod_data_df['strat_return'] = mod_data_df.apply(go_compute, axis=1)
+
+    mod_data_df['strat_return'] = mod_data_df['strat_return'].fillna(0.)
+    me_strat = 'strat'
+    mod_data_df[me_strat] = np.cumprod(1 + mod_data_df['strat_return'].values)
+
+    if plot_html:
+        fig = realtime_plotting_utility.plot_multiple_time_series(
+            data_df=mod_data_df[[me_strat]],
+            logy=True, split=False,
+            put_on_same_scale=False,
+            title=f'Perf{first_token}{second_token}')
+        fig.show()
+
+    title = f'DUAL_RSI_{moderation}'
+    backtest_df = mod_data_df[[me_strat]].copy()
+    backtest_df.columns = ['strat']
+    strat_data_df = backtest_df[['strat']].copy()
+
+    daily_df = riskmetrics.filter_daily(strat_data_df)
+    daily_df = daily_df.dropna()
+
+    if compute_lo_strat:
+        title = 'BTC_weekly_rsilo'
+        backtest_df = daily_df[['strat']].copy()
+        whole_kpis = True
+        if whole_kpis:
+            strat_data_df = backtest_df[['strat']].copy()
+            from cryptotoolbox.analyzer import market
+
+            # strat_data_df = strat_data_df[strat_data_df.index >= '2021-02-01'].copy()
+            ma = market.MarketAnalyzer(strat_data_df, hourlyze=True)
+
+            print('computing kpis')
+            kpi_df = ma.get_kpi().T
+            kpi_df = kpi_df.dropna()
+            kpi_df = kpi_df[~np.isnan(kpi_df.sharpe)]
+            print(kpi_df.head())
+
+        backtest_df.columns = ['strat']
+        riskmetrics.compute_plot_excel_kpis_meterics_metrics(backtest_df, title,
+                                                             local_root_directory,
+                                                             plot_html=True,
+                                                             write_file=True)
+    return mod_data_df.copy()
+
+def compute_alpha_signal_optimal_lo(df=None, core_tokens=['BTC', 'ETH'], extra_tokens=['AVAX'],     p1btc = 1.01, p2btc = 1.05, p1eth = 1.01, p2eth = 1.05, tone = 14,ttwo = 23, rebalancing_day=6,lo_sig_suffix='LO',compute_lo_strat=True, plot_html=False, whole_kpis=False,local_root_directory='None'):
     all_tokens = core_tokens + extra_tokens
     df['weekday'] = df.index.weekday
     df['rebalance'] = df.index.weekday == rebalancing_day
